package fr.inria.phoenix.diasuite.framework.context.averageoccupancy;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fr.inria.phoenix.diasuite.framework.datatype.presence.PresenceWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.OutputCollector;

import fr.inria.diagen.core.mapreduce.MapCollector;
import fr.inria.diagen.core.mapreduce.MapReduce;
import fr.inria.diagen.core.mapreduce.ReduceCollector;
import fr.inria.diagen.core.mapreduce.Utils;

import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;
import fr.inria.phoenix.diasuite.framework.datatype.presence.Presence;

public class AverageOccupancyAdapter {
	
	public AverageOccupancyAdapter(
			MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Presence, ParkingLotEnum, Integer> adaptee) {
		this.adaptee = adaptee;
	}
	
	private MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Presence, ParkingLotEnum, Integer> adaptee;
	
	public void doMap(LongWritable key, Text value,
			final OutputCollector<Text, PresenceWritable> output) {
		
		/*********** transform hadoop types to diaswarm ***********/
		
		ParkingLotEnum parkingLot = ParkingLotEnum.valueOf(Utils
				.getFieldFromJSON("parkingLot", value));
		Boolean presence = Boolean.valueOf(Utils
				.getFieldFromJSON("presence", value));
		String[] args = new String[] { Utils
					.getFieldFromJSON("position", value) };
		/**********************************************************/
		
		adaptee.map(parkingLot, presence,
				new MapCollector<ParkingLotEnum, Presence>() {
					
					@Override
					public void emitMap(ParkingLotEnum key, Presence value) {
						Text mapKeyOut = Utils.getTextWritable(key.toString());
						PresenceWritable mapValueOut = new PresenceWritable(value);
								
						try {
							output.collect(mapKeyOut, mapValueOut);
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				}, args);
	}
	
	public void doReduce(Text key, Iterator<PresenceWritable> values,
			final OutputCollector<Text, IntWritable> output) {
		
		/*********** transform hadoop types to diaswarm ***********/
		ParkingLotEnum parkingLot = ParkingLotEnum.valueOf(key.toString());
		List<Presence> presenceValues = new ArrayList<Presence>();
		 
		while (values.hasNext()) {
			presenceValues.add(values.next().get());
		}
		/**********************************************************/
		adaptee.reduce(parkingLot, presenceValues,
				new ReduceCollector<ParkingLotEnum, Integer>() {
					
					@Override
					public void emitReduce(ParkingLotEnum key, Integer value) {
						Text reduceKeyOut = Utils.getTextWritable(key
								.toString());
						IntWritable reduceValueOut = Utils
								.getIntWritable(value);
								
						try {
							output.collect(reduceKeyOut, reduceValueOut);
						} catch (IOException e) {
							// TODO Auto-generated catch block 
							e.printStackTrace();
						}
					}
				});
	}
}
