package fr.inria.phoenix.diasuite.framework.context.averageoccupancy;

import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import fr.inria.phoenix.diasuite.framework.datatype.presence.PresenceWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.hadoop.mapred.TextOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;
import fr.inria.phoenix.diasuite.framework.datatype.presence.Presence;


import fr.inria.diagen.core.mapreduce.MapReduce;
import fr.inria.diagen.log.DiaLog;

public class AverageOccupancyJob extends Configured implements Tool {
	
	public static AverageOccupancyAdapter jobLauncher;
	
	public AverageOccupancyJob(
			MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Presence, ParkingLotEnum, Integer> context) {
		try {
			jobLauncher = new AverageOccupancyAdapter(context);
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	public static class AverageOccupancyMap extends MapReduceBase implements
			Mapper<LongWritable, Text, Text, PresenceWritable> {
				
		@Override
		public void map(LongWritable key, Text value,
				OutputCollector<Text, PresenceWritable> output, Reporter reporter)
				throws IOException {
			jobLauncher.doMap(key, value, output);
		}
	}
	
	public static class AverageOccupancyReduce extends MapReduceBase
			implements Reducer<Text, PresenceWritable, Text, IntWritable> {
		@Override
		public void reduce(Text key, Iterator<PresenceWritable> values,
				OutputCollector<Text, IntWritable> output, Reporter reporter)
				throws IOException {
			jobLauncher.doReduce(key, values, output);
	}
}

	@Override
	public int run(String[] args) throws Exception {
		Logger.getRootLogger().getLoggerRepository().resetConfiguration();
		Logger.getRootLogger().addAppender(
				DiaLog.getFileAppender("AverageOccupancy",
						"AverageOccupancy.log", Level.DEBUG, true));
	
		JobConf conf = new JobConf(getConf(), AverageOccupancyJob.class);
		conf.setJobName("AverageOccupancy");
	
		/* set mapred output types */
		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(IntWritable.class);
	
		/* set map output types */
		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(PresenceWritable.class);
	
		conf.setMapperClass(AverageOccupancyMap.class);
		conf.setReducerClass(AverageOccupancyReduce.class);
	
		conf.setInputFormat(TextInputFormat.class);
		conf.setOutputFormat(TextOutputFormat.class);
	
		FileInputFormat.setInputPaths(conf, new Path(args[0]));
		FileOutputFormat.setOutputPath(conf, new Path(args[1]));
	
		JobClient.runJob(conf);
		return 0;
	}
}
