package fr.inria.phoenix.diasuite.framework.context.parkingavailability;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;
import fr.inria.diagen.core.service.buffer.Buffer;
import java.util.HashMap;
import java.util.Map;
import fr.inria.diagen.core.network.DataDeliveryEnum;
import fr.inria.diagen.core.network.DataDeliveryModel;
import fr.inria.diagen.core.service.local.ServiceProperties;


import java.util.List;
import fr.inria.diagen.core.network.TimeUnit;
import fr.inria.diagen.core.service.buffer.DeviceSourceBuffer;


import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;        	

import fr.inria.diagen.core.mapreduce.MapReduce;
@SuppressWarnings("all")
public abstract class AbstractParkingAvailability extends Service implements MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Boolean, ParkingLotEnum, Integer> {

	/**================== CLASS ATTRIBUTES ==================**/
	private java.util.List<fr.inria.phoenix.diasuite.framework.datatype.availability.Availability> contextValue;
	
		private static final Map<String, Object> deliveryModels;
		
		static {
			deliveryModels = new HashMap<String, Object>();
			
			deliveryModels.put("presence",
				new DataDeliveryModel.Builder()
					.deliveryModel(DataDeliveryEnum.PERIODIC)
						.period(30).timeUnit(TimeUnit.S).valueCount(1).build());
		}
	/*=======================================================*/
	
	
	/**================= CONTEXT CONSTRUCTOR ================**/
	public AbstractParkingAvailability(ServiceConfiguration serviceConfiguration) {
		super("/Context/ParkingAvailability/", serviceConfiguration);
		
		
	addToOutProperties(ServiceProperties.DELIVERY.toString(),
			deliveryModels);
	}
    /*=======================================================*/
    
    
    /**============== METHODS FOR CONTEXT VALUE =============**/
	// Code relative to the return value of the context        
	    
	    private void setParkingAvailability(java.util.List<fr.inria.phoenix.diasuite.framework.datatype.availability.Availability> newContextValue) {
	        contextValue = newContextValue;
	        getProcessor().publishValue(getOutProperties(), "ParkingAvailability", newContextValue);
	    }
	    
	    /**
	     * Get the last value of the context
	     * 
	     * @return the latest value published by the context
	     */
	    protected final java.util.List<fr.inria.phoenix.diasuite.framework.datatype.availability.Availability> getLastValue() {
	        return contextValue;
	    }
    /*=======================================================*/
    
    
    /**============ CONTEXT RETURN VALUE CLASS ==============**/
            // End of code relative to the return value of the context
    /*=======================================================*/
    
    /**============ INTERACTION CONTRACT METHOD =============**/
    protected abstract java.util.List<fr.inria.phoenix.diasuite.framework.datatype.availability.Availability> onPeriodicPresenceFromPresenceSensor(Map<ParkingLotEnum, Integer> presenceByParkingLotMap);
    /*=======================================================*/
    
    /**============== DEVICE SUBSCRIPTION CODE ==============**/
    
    	protected final PresenceSensorSubscriber presenceSensorSubscriber = new PresenceSensorSubscriber(this);        
    	
    	protected final static class PresenceSensorSubscriber {
    	    private Service serviceParent;
    	    
    	    private PresenceSensorSubscriber(Service serviceParent) {
    	        super();
    	        this.serviceParent = serviceParent;
    	    }
    	    
    	    private PresenceSensorComposite instantiateComposite() {
    	        return new PresenceSensorComposite(serviceParent);
    	    }
    	    
    	    /**
    	     * Returns a composite of all accessible <code>PresenceSensor</code> devices
    	     * 
    	     * @return a {@link PresenceSensorComposite} object composed of all discoverable <code>PresenceSensor</code>
    	     */
    	    public PresenceSensorComposite all() {
    	        return instantiateComposite();
    	    }
    	    
    	    /**
    	     * Returns a proxy to one out of all accessible <code>PresenceSensor</code> devices
    	     * 
    	     * @return a {@link PresenceSensorProxy} object pointing to a random discoverable <code>PresenceSensor</code> device
    	     */
    	    public PresenceSensorProxy anyOne() {
    	        return all().anyOne();
    	    }
    	    
    	    /**
    	     * Returns a composite of all accessible <code>PresenceSensor</code> devices whose attribute <code>id</code> matches a given value.
    	     * 
    	     * @return a {@link PresenceSensorComposite} object composed of all matching <code>PresenceSensor</code> devices
    	     */
    	    public PresenceSensorComposite whereId(java.lang.String id) throws CompositeException {
    	        return instantiateComposite().andId(id);
    	    }
    	    
    	    /**
    	     * Returns a composite of all accessible <code>PresenceSensor</code> devices whose attribute <code>parkingLot</code> matches a given value.
    	     * 
    	     * @return a {@link PresenceSensorComposite} object composed of all matching <code>PresenceSensor</code> devices
    	     */
    	    public PresenceSensorComposite whereParkingLot(fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum parkingLot) throws CompositeException {
    	        return instantiateComposite().andParkingLot(parkingLot);
    	    }
    	}
    	protected final static class PresenceSensorComposite extends fr.inria.diagen.core.service.composite.Composite<PresenceSensorProxy> {
    	    private PresenceSensorComposite(Service serviceParent) {
    	        super(serviceParent, "/Device/Device/PresenceSensor/");
    	    }
    	    
    	    @Override
    	    protected PresenceSensorProxy instantiateProxy(RemoteServiceInfo rsi) {
    	        return new PresenceSensorProxy(service, rsi);
    	    }
    	    /**
    	     * Returns this composite in which a filter was set to the attribute <code>id</code>.
    	     * 
    	     * @param id The <code>id<code> attribute value to match.
    	     * @return this {@link PresenceSensorComposite}, filtered using the attribute <code></code>.
    	     */
    	    public PresenceSensorComposite andId(java.lang.String id) throws CompositeException {
    	    	filterByAttribute("id", id);
    	    	return this;
    	    }
    	    /**
    	     * Returns this composite in which a filter was set to the attribute <code>parkingLot</code>.
    	     * 
    	     * @param parkingLot The <code>parkingLot<code> attribute value to match.
    	     * @return this {@link PresenceSensorComposite}, filtered using the attribute <code></code>.
    	     */
    	    public PresenceSensorComposite andParkingLot(fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum parkingLot) throws CompositeException {
    	    	filterByAttribute("parkingLot", parkingLot);
    	    	return this;
    	    }
    	
    	    /**
    	     * Subscribes to the <code>presence</code> source. After a call to this method, the context will be notified when a
    	     * <code>PresenceSensor</code> device of this composite publishes a value on its <code>presence</code> source.
    	     */
    	    public void subscribePresence() {
    	    	subscribeValue("presence");
    	    }
    	    
    	    /**
    	     * Unsubscribes from the <code>presence</code> source. After a call to this method, the context will no more be notified
    	     * when a <code>PresenceSensor</code> device of this composite publishes a value on its <code>presence</code> source.
    	     */
    	    public void unsubscribePresence() {
    	    	unsubscribeValue("presence");
    	    }
    	}
    	protected final static class PresenceSensorProxy extends Proxy {
    	    private PresenceSensorProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
    	        super(service, remoteServiceInfo);
    	    }
    	    /**
    	     * Subscribes to the <code>presence</code> source. After a call to this method, the context will be notified when a
    	     * <code>PresenceSensor</code> device of this composite publishes a value on its <code>presence</code> source.
    	     */
    	    public void subscribePresence() {
    	    	subscribeValue("presence");
    	    }
    	    
    	    /**
    	     * Unsubscribes from the <code>presence</code> source. After a call to this method, the context will no more be notified
    	     * when a <code>PresenceSensor</code> device of this composite publishes a value on its <code>presence</code> source.
    	     */
    	    public void unsubscribePresence() {
    	    	unsubscribeValue("presence");
    	    }
    	    
    	    /**
    	     * @return the value of the <code>id</code> attribute
    	     */
    	    public java.lang.String id() {
    	        return (java.lang.String) callGetValue("id");
    	    }
    	    
    	    /**
    	     * @return the value of the <code>parkingLot</code> attribute
    	     */
    	    public fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum parkingLot() {
    	        return (fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum) callGetValue("parkingLot");
    	    }
    	}
    	// End of discover part for PresenceSensor devices
    /*=======================================================*/
    
    /**================ DEVICE DISCOVERY CODE ===============**/
    
    /*=======================================================*/
    
    /**=================== SERVICE METHODS ==================**/
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
    	postInitialize();
    }
    	
    @Override protected void postInitialize() {
    	// Default implementation of post initialize: subscribe to all required devices 
    	presenceSensorSubscriber.all().subscribePresence();
    	// subscribe to presence from all PresenceSensor devices
    }
    
    
        @Override
        public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        	
        	if (eventName.equals("presence") && source.isCompatible("/Device/Device/PresenceSensor/")) {
        		Map<ParkingLotEnum, Integer> presenceMap = Map.class.cast(value);
        		
        		setParkingAvailability(onPeriodicPresenceFromPresenceSensor(presenceMap));
        	}
        }
        
    
    
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("ParkingAvailability")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    /*=======================================================*/
}
