package fr.inria.phoenix.diasuite.framework.context.parkingavailability;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.hadoop.io.BooleanWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.OutputCollector;

import fr.inria.diagen.core.mapreduce.MapCollector;
import fr.inria.diagen.core.mapreduce.MapReduce;
import fr.inria.diagen.core.mapreduce.ReduceCollector;
import fr.inria.diagen.core.mapreduce.Utils;

import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;

public class ParkingAvailabilityAdapter {
	
	public ParkingAvailabilityAdapter(
			MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Boolean, ParkingLotEnum, Integer> adaptee) {
		this.adaptee = adaptee;
	}
	
	private MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Boolean, ParkingLotEnum, Integer> adaptee;
	
	public void doMap(LongWritable key, Text value,
			final OutputCollector<Text, BooleanWritable> output) {
		
		/*********** transform hadoop types to diaswarm ***********/
		
		ParkingLotEnum parkingLot = ParkingLotEnum.valueOf(Utils
				.getFieldFromJSON("parkingLot", value));
		Boolean presence = Boolean.valueOf(Utils
				.getFieldFromJSON("presence", value));
		String[] args = new String[] { Utils
					.getFieldFromJSON("position", value) };
		/**********************************************************/
		
		adaptee.map(parkingLot, presence,
				new MapCollector<ParkingLotEnum, Boolean>() {
					
					@Override
					public void emitMap(ParkingLotEnum key, Boolean value) {
						Text mapKeyOut = Utils.getTextWritable(key.toString());
						BooleanWritable mapValueOut = new BooleanWritable(value);
								
						try {
							output.collect(mapKeyOut, mapValueOut);
						} catch (IOException e) {
							e.printStackTrace();
						}
					}
				}, args);
	}
	
	public void doReduce(Text key, Iterator<BooleanWritable> values,
			final OutputCollector<Text, IntWritable> output) {
		
		/*********** transform hadoop types to diaswarm ***********/
		ParkingLotEnum parkingLot = ParkingLotEnum.valueOf(key.toString());
		List<Boolean> presenceValues = new ArrayList<Boolean>();
		 
		while (values.hasNext()) {
			presenceValues.add(values.next().get());
		}
		/**********************************************************/
		adaptee.reduce(parkingLot, presenceValues,
				new ReduceCollector<ParkingLotEnum, Integer>() {
					
					@Override
					public void emitReduce(ParkingLotEnum key, Integer value) {
						Text reduceKeyOut = Utils.getTextWritable(key
								.toString());
						IntWritable reduceValueOut = Utils
								.getIntWritable(value);
								
						try {
							output.collect(reduceKeyOut, reduceValueOut);
						} catch (IOException e) {
							// TODO Auto-generated catch block 
							e.printStackTrace();
						}
					}
				});
	}
}
