package fr.inria.phoenix.diasuite.framework.context.parkingavailability;

import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.BooleanWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.hadoop.mapred.TextOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;


import fr.inria.diagen.core.mapreduce.MapReduce;
import fr.inria.diagen.log.DiaLog;

public class ParkingAvailabilityJob extends Configured implements Tool {
	
	public static ParkingAvailabilityAdapter jobLauncher;
	
	public ParkingAvailabilityJob(
			MapReduce<ParkingLotEnum, Boolean, ParkingLotEnum, Boolean, ParkingLotEnum, Integer> context) {
		try {
			jobLauncher = new ParkingAvailabilityAdapter(context);
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	
	public static class ParkingAvailabilityMap extends MapReduceBase implements
			Mapper<LongWritable, Text, Text, BooleanWritable> {
				
		@Override
		public void map(LongWritable key, Text value,
				OutputCollector<Text, BooleanWritable> output, Reporter reporter)
				throws IOException {
			jobLauncher.doMap(key, value, output);
		}
	}
	
	public static class ParkingAvailabilityReduce extends MapReduceBase
			implements Reducer<Text, BooleanWritable, Text, IntWritable> {
		@Override
		public void reduce(Text key, Iterator<BooleanWritable> values,
				OutputCollector<Text, IntWritable> output, Reporter reporter)
				throws IOException {
			jobLauncher.doReduce(key, values, output);
	}
}

	@Override
	public int run(String[] args) throws Exception {
		Logger.getRootLogger().getLoggerRepository().resetConfiguration();
		Logger.getRootLogger().addAppender(
				DiaLog.getFileAppender("ParkingAvailability",
						"ParkingAvailability.log", Level.DEBUG, true));
	
		JobConf conf = new JobConf(getConf(), ParkingAvailabilityJob.class);
		conf.setJobName("ParkingAvailability");
	
		/* set mapred output types */
		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(IntWritable.class);
	
		/* set map output types */
		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(BooleanWritable.class);
	
		conf.setMapperClass(ParkingAvailabilityMap.class);
		conf.setReducerClass(ParkingAvailabilityReduce.class);
	
		conf.setInputFormat(TextInputFormat.class);
		conf.setOutputFormat(TextOutputFormat.class);
	
		FileInputFormat.setInputPaths(conf, new Path(args[0]));
		FileOutputFormat.setOutputPath(conf, new Path(args[1]));
	
		JobClient.runJob(conf);
		return 0;
	}
}
