package fr.inria.phoenix.diasuite.framework.controller.cityentrancepanelcontroller;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;
import fr.inria.diagen.core.network.DataDeliveryEnum;
import fr.inria.diagen.core.network.DataDeliveryModel;
import fr.inria.diagen.core.service.local.ServiceProperties;

import java.util.Map;
import java.util.HashMap;

import fr.inria.phoenix.diasuite.framework.context.parkingsuggestion.ParkingSuggestionValue;

@SuppressWarnings("all")
public abstract class AbstractCityEntrancePanelController extends Service {
	
	private static final Map<String, Object> deliveryModels;
		
		static {
			deliveryModels = new HashMap<String, Object>();
			deliveryModels.put("ParkingSuggestion",
						new DataDeliveryModel.Builder()
								.deliveryModel(DataDeliveryEnum.EVENT_DRIVEN)
								.build());
		}
    
    public AbstractCityEntrancePanelController(ServiceConfiguration serviceConfiguration) {
        super("/Controller/CityEntrancePanelController/", serviceConfiguration);
        addToOutProperties(ServiceProperties.DELIVERY.toString(),
    				deliveryModels);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("ParkingSuggestion", "/Context/ParkingSuggestion/"); // subscribe to ParkingSuggestion context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("ParkingSuggestion") && source.isCompatible("/Context/ParkingSuggestion/")) {
            ParkingSuggestionValue parkingSuggestionValue = new ParkingSuggestionValue((java.util.List<fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum>) value);
            
            onParkingSuggestion(parkingSuggestionValue, new Discover());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>ParkingSuggestion</code> context publishes a value.
     * 
     * <pre>
     * </pre>
     * 
     * @param parkingSuggestion the value of the <code>ParkingSuggestion</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onParkingSuggestion(ParkingSuggestionValue parkingSuggestion, Discover discover);
    
    // End of interaction contract implementation
    
    // Discover object for ParkingSuggestion
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * 
     * </code>
     */
    protected final class Discover {
        private final CityEntrancePanelDiscovererForParkingSuggestion cityEntrancePanelDiscoverer = new CityEntrancePanelDiscovererForParkingSuggestion(AbstractCityEntrancePanelController.this);
        
        /**
         * @return a {@link CityEntrancePanelDiscovererForParkingSuggestion} object to discover <code>CityEntrancePanel</code> devices
         */
        public CityEntrancePanelDiscovererForParkingSuggestion cityEntrancePanels() {
            return cityEntrancePanelDiscoverer;
        }
    }
    
    protected final static class CityEntrancePanelDiscovererForParkingSuggestion {
        private Service serviceParent;
        
        private CityEntrancePanelDiscovererForParkingSuggestion(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private CityEntrancePanelCompositeForParkingSuggestion instantiateComposite() {
            return new CityEntrancePanelCompositeForParkingSuggestion(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>CityEntrancePanel</code> devices
         * 
         * @return a {@link CityEntrancePanelCompositeForParkingSuggestion} object composed of all discoverable <code>CityEntrancePanel</code>
         */
        public CityEntrancePanelCompositeForParkingSuggestion all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>CityEntrancePanel</code> devices
         * 
         * @return a {@link CityEntrancePanelProxyForParkingSuggestion} object pointing to a random discoverable <code>CityEntrancePanel</code> device
         */
        public CityEntrancePanelProxyForParkingSuggestion anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>CityEntrancePanel</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @return a {@link CityEntrancePanelCompositeForParkingSuggestion} object composed of all matching <code>CityEntrancePanel</code> devices
         */
        public CityEntrancePanelCompositeForParkingSuggestion whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>CityEntrancePanel</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @return a {@link CityEntrancePanelCompositeForParkingSuggestion} object composed of all matching <code>CityEntrancePanel</code> devices
         */
        public CityEntrancePanelCompositeForParkingSuggestion whereLocation(fr.inria.phoenix.diasuite.framework.datatype.cityentranceenum.CityEntranceEnum location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
    }
    
    protected final static class CityEntrancePanelCompositeForParkingSuggestion extends fr.inria.diagen.core.service.composite.Composite<CityEntrancePanelProxyForParkingSuggestion> {
        private CityEntrancePanelCompositeForParkingSuggestion(Service serviceParent) {
            super(serviceParent, "/Device/DisplayPanel/CityEntrancePanel/");
        }
        
        @Override
        protected CityEntrancePanelProxyForParkingSuggestion instantiateProxy(RemoteServiceInfo rsi) {
            return new CityEntrancePanelProxyForParkingSuggestion(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @return this {@link CityEntrancePanelCompositeForParkingSuggestion}, filtered using the attribute <code>id</code>.
         */
        public CityEntrancePanelCompositeForParkingSuggestion andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @return this {@link CityEntrancePanelCompositeForParkingSuggestion}, filtered using the attribute <code>location</code>.
         */
        public CityEntrancePanelCompositeForParkingSuggestion andLocation(fr.inria.phoenix.diasuite.framework.datatype.cityentranceenum.CityEntranceEnum location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * 
         * <p>
         */
        public void update(java.lang.String status) throws InvocationException {
            launchDiscovering();
            for (CityEntrancePanelProxyForParkingSuggestion proxy : proxies) {
                proxy.update(status);
            }
        }
    }
    
    protected final static class CityEntrancePanelProxyForParkingSuggestion extends Proxy {
        private CityEntrancePanelProxyForParkingSuggestion(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         */
        public void update(java.lang.String status) throws InvocationException {
            callOrder("update", status);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public fr.inria.phoenix.diasuite.framework.datatype.cityentranceenum.CityEntranceEnum location() {
            return (fr.inria.phoenix.diasuite.framework.datatype.cityentranceenum.CityEntranceEnum) callGetValue("location");
        }
    }
    // End of discover object for ParkingSuggestion
}
