package fr.inria.phoenix.diasuite.framework.controller.messengercontroller;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;
import fr.inria.diagen.core.network.DataDeliveryEnum;
import fr.inria.diagen.core.network.DataDeliveryModel;
import fr.inria.diagen.core.service.local.ServiceProperties;

import java.util.Map;
import java.util.HashMap;

import fr.inria.phoenix.diasuite.framework.context.averageoccupancy.AverageOccupancyValue;

@SuppressWarnings("all")
public abstract class AbstractMessengerController extends Service {
	
	private static final Map<String, Object> deliveryModels;
		
		static {
			deliveryModels = new HashMap<String, Object>();
			deliveryModels.put("AverageOccupancy",
						new DataDeliveryModel.Builder()
								.deliveryModel(DataDeliveryEnum.EVENT_DRIVEN)
								.build());
		}
    
    public AbstractMessengerController(ServiceConfiguration serviceConfiguration) {
        super("/Controller/MessengerController/", serviceConfiguration);
        addToOutProperties(ServiceProperties.DELIVERY.toString(),
    				deliveryModels);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("AverageOccupancy", "/Context/AverageOccupancy/"); // subscribe to AverageOccupancy context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("AverageOccupancy") && source.isCompatible("/Context/AverageOccupancy/")) {
            AverageOccupancyValue averageOccupancyValue = new AverageOccupancyValue((java.util.List<fr.inria.phoenix.diasuite.framework.datatype.parkingoccupancy.ParkingOccupancy>) value);
            
            onAverageOccupancy(averageOccupancyValue, new Discover());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>AverageOccupancy</code> context publishes a value.
     * 
     * <pre>
     * </pre>
     * 
     * @param averageOccupancy the value of the <code>AverageOccupancy</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onAverageOccupancy(AverageOccupancyValue averageOccupancy, Discover discover);
    
    // End of interaction contract implementation
    
    // Discover object for AverageOccupancy
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * 
     * </code>
     */
    protected final class Discover {
        private final MessengerDiscovererForAverageOccupancy messengerDiscoverer = new MessengerDiscovererForAverageOccupancy(AbstractMessengerController.this);
        
        /**
         * @return a {@link MessengerDiscovererForAverageOccupancy} object to discover <code>Messenger</code> devices
         */
        public MessengerDiscovererForAverageOccupancy messengers() {
            return messengerDiscoverer;
        }
    }
    
    protected final static class MessengerDiscovererForAverageOccupancy {
        private Service serviceParent;
        
        private MessengerDiscovererForAverageOccupancy(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private MessengerCompositeForAverageOccupancy instantiateComposite() {
            return new MessengerCompositeForAverageOccupancy(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices
         * 
         * @return a {@link MessengerCompositeForAverageOccupancy} object composed of all discoverable <code>Messenger</code>
         */
        public MessengerCompositeForAverageOccupancy all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Messenger</code> devices
         * 
         * @return a {@link MessengerProxyForAverageOccupancy} object pointing to a random discoverable <code>Messenger</code> device
         */
        public MessengerProxyForAverageOccupancy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @return a {@link MessengerCompositeForAverageOccupancy} object composed of all matching <code>Messenger</code> devices
         */
        public MessengerCompositeForAverageOccupancy whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    protected final static class MessengerCompositeForAverageOccupancy extends fr.inria.diagen.core.service.composite.Composite<MessengerProxyForAverageOccupancy> {
        private MessengerCompositeForAverageOccupancy(Service serviceParent) {
            super(serviceParent, "/Device/Device/Messenger/");
        }
        
        @Override
        protected MessengerProxyForAverageOccupancy instantiateProxy(RemoteServiceInfo rsi) {
            return new MessengerProxyForAverageOccupancy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @return this {@link MessengerCompositeForAverageOccupancy}, filtered using the attribute <code>id</code>.
         */
        public MessengerCompositeForAverageOccupancy andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * 
         * <p>
         */
        public void sendMessage(java.lang.String message) throws InvocationException {
            launchDiscovering();
            for (MessengerProxyForAverageOccupancy proxy : proxies) {
                proxy.sendMessage(message);
            }
        }
    }
    
    protected final static class MessengerProxyForAverageOccupancy extends Proxy {
        private MessengerProxyForAverageOccupancy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         */
        public void sendMessage(java.lang.String message) throws InvocationException {
            callOrder("sendMessage", message);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for AverageOccupancy
}
