package fr.inria.phoenix.diasuite.framework.controller.parkingentrancepanelcontroller;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;
import fr.inria.diagen.core.network.DataDeliveryEnum;
import fr.inria.diagen.core.network.DataDeliveryModel;
import fr.inria.diagen.core.service.local.ServiceProperties;

import java.util.Map;
import java.util.HashMap;

import fr.inria.phoenix.diasuite.framework.context.parkingavailability.ParkingAvailabilityValue;

@SuppressWarnings("all")
public abstract class AbstractParkingEntrancePanelController extends Service {
	
	private static final Map<String, Object> deliveryModels;
		
		static {
			deliveryModels = new HashMap<String, Object>();
			deliveryModels.put("ParkingAvailability",
						new DataDeliveryModel.Builder()
								.deliveryModel(DataDeliveryEnum.EVENT_DRIVEN)
								.build());
		}
    
    public AbstractParkingEntrancePanelController(ServiceConfiguration serviceConfiguration) {
        super("/Controller/ParkingEntrancePanelController/", serviceConfiguration);
        addToOutProperties(ServiceProperties.DELIVERY.toString(),
    				deliveryModels);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("ParkingAvailability", "/Context/ParkingAvailability/"); // subscribe to ParkingAvailability context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("ParkingAvailability") && source.isCompatible("/Context/ParkingAvailability/")) {
            ParkingAvailabilityValue parkingAvailabilityValue = new ParkingAvailabilityValue((java.util.List<fr.inria.phoenix.diasuite.framework.datatype.availability.Availability>) value);
            
            onParkingAvailability(parkingAvailabilityValue, new Discover());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>ParkingAvailability</code> context publishes a value.
     * 
     * <pre>
     * </pre>
     * 
     * @param parkingAvailability the value of the <code>ParkingAvailability</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onParkingAvailability(ParkingAvailabilityValue parkingAvailability, Discover discover);
    
    // End of interaction contract implementation
    
    // Discover object for ParkingAvailability
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * 
     * </code>
     */
    protected final class Discover {
        private final ParkingEntrancePanelDiscovererForParkingAvailability parkingEntrancePanelDiscoverer = new ParkingEntrancePanelDiscovererForParkingAvailability(AbstractParkingEntrancePanelController.this);
        
        /**
         * @return a {@link ParkingEntrancePanelDiscovererForParkingAvailability} object to discover <code>ParkingEntrancePanel</code> devices
         */
        public ParkingEntrancePanelDiscovererForParkingAvailability parkingEntrancePanels() {
            return parkingEntrancePanelDiscoverer;
        }
    }
    
    protected final static class ParkingEntrancePanelDiscovererForParkingAvailability {
        private Service serviceParent;
        
        private ParkingEntrancePanelDiscovererForParkingAvailability(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ParkingEntrancePanelCompositeForParkingAvailability instantiateComposite() {
            return new ParkingEntrancePanelCompositeForParkingAvailability(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>ParkingEntrancePanel</code> devices
         * 
         * @return a {@link ParkingEntrancePanelCompositeForParkingAvailability} object composed of all discoverable <code>ParkingEntrancePanel</code>
         */
        public ParkingEntrancePanelCompositeForParkingAvailability all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>ParkingEntrancePanel</code> devices
         * 
         * @return a {@link ParkingEntrancePanelProxyForParkingAvailability} object pointing to a random discoverable <code>ParkingEntrancePanel</code> device
         */
        public ParkingEntrancePanelProxyForParkingAvailability anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>ParkingEntrancePanel</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @return a {@link ParkingEntrancePanelCompositeForParkingAvailability} object composed of all matching <code>ParkingEntrancePanel</code> devices
         */
        public ParkingEntrancePanelCompositeForParkingAvailability whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>ParkingEntrancePanel</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @return a {@link ParkingEntrancePanelCompositeForParkingAvailability} object composed of all matching <code>ParkingEntrancePanel</code> devices
         */
        public ParkingEntrancePanelCompositeForParkingAvailability whereLocation(fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
    }
    
    protected final static class ParkingEntrancePanelCompositeForParkingAvailability extends fr.inria.diagen.core.service.composite.Composite<ParkingEntrancePanelProxyForParkingAvailability> {
        private ParkingEntrancePanelCompositeForParkingAvailability(Service serviceParent) {
            super(serviceParent, "/Device/DisplayPanel/ParkingEntrancePanel/");
        }
        
        @Override
        protected ParkingEntrancePanelProxyForParkingAvailability instantiateProxy(RemoteServiceInfo rsi) {
            return new ParkingEntrancePanelProxyForParkingAvailability(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @return this {@link ParkingEntrancePanelCompositeForParkingAvailability}, filtered using the attribute <code>id</code>.
         */
        public ParkingEntrancePanelCompositeForParkingAvailability andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @return this {@link ParkingEntrancePanelCompositeForParkingAvailability}, filtered using the attribute <code>location</code>.
         */
        public ParkingEntrancePanelCompositeForParkingAvailability andLocation(fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * 
         * <p>
         */
        public void update(java.lang.String status) throws InvocationException {
            launchDiscovering();
            for (ParkingEntrancePanelProxyForParkingAvailability proxy : proxies) {
                proxy.update(status);
            }
        }
    }
    
    protected final static class ParkingEntrancePanelProxyForParkingAvailability extends Proxy {
        private ParkingEntrancePanelProxyForParkingAvailability(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         */
        public void update(java.lang.String status) throws InvocationException {
            callOrder("update", status);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum location() {
            return (fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum) callGetValue("location");
        }
    }
    // End of discover object for ParkingAvailability
}
