package fr.inria.phoenix.diasuite.framework.deployment;

import fr.inria.diagen.core.deploy.AbstractDeploy;

import fr.inria.phoenix.diasuite.framework.context.parkingavailability.AbstractParkingAvailability;
import fr.inria.phoenix.diasuite.framework.context.parkingusagepattern.AbstractParkingUsagePattern;
import fr.inria.phoenix.diasuite.framework.context.averageoccupancy.AbstractAverageOccupancy;
import fr.inria.phoenix.diasuite.framework.context.parkingsuggestion.AbstractParkingSuggestion;

import fr.inria.diagen.core.service.buffer.DeviceSourceBuffer;

import fr.inria.phoenix.diasuite.framework.context.parkingavailability.ParkingAvailabilityJob;
import org.apache.hadoop.util.Tool;
import fr.inria.diagen.core.network.TimeUnit;
import fr.inria.diagen.core.mapreduce.MapReduceConfiguration;
import fr.inria.diagen.core.service.local.ServiceProperties;
import fr.inria.phoenix.diasuite.framework.context.averageoccupancy.AverageOccupancyJob;

import fr.inria.phoenix.diasuite.framework.controller.parkingentrancepanelcontroller.AbstractParkingEntrancePanelController;
import fr.inria.phoenix.diasuite.framework.controller.cityentrancepanelcontroller.AbstractCityEntrancePanelController;
import fr.inria.phoenix.diasuite.framework.controller.messengercontroller.AbstractMessengerController;

/**
 * This class should be implemented to bind the implementation of the various components
 */
public abstract class AppComponentBinder extends AbstractDeploy {

    // Context instances
    private AbstractParkingAvailability parkingAvailabilityInstance = null;
    private AbstractParkingUsagePattern parkingUsagePatternInstance = null;
    private AbstractAverageOccupancy averageOccupancyInstance = null;
    private AbstractParkingSuggestion parkingSuggestionInstance = null;

    // Controller instances
    private AbstractParkingEntrancePanelController parkingEntrancePanelControllerInstance = null;
    private AbstractCityEntrancePanelController cityEntrancePanelControllerInstance = null;
    private AbstractMessengerController messengerControllerInstance = null;
    
    @Override
    public void deployAll() {
    	
    	initializeSourceBuffer("presence", 
    							getPresenceFromPresenceSensorBufferClass());
        // Initialization of contexts
        if (parkingAvailabilityInstance == null) {
            parkingAvailabilityInstance = getServiceInstance(getParkingAvailabilityClass());
        	setParkingAvailabilityMapReduceConfig();
        }    
        if (parkingUsagePatternInstance == null) {
            parkingUsagePatternInstance = getServiceInstance(getParkingUsagePatternClass());
        }    
        if (averageOccupancyInstance == null) {
            averageOccupancyInstance = getServiceInstance(getAverageOccupancyClass());
        	setAverageOccupancyMapReduceConfig();
        }    
        if (parkingSuggestionInstance == null) {
            parkingSuggestionInstance = getServiceInstance(getParkingSuggestionClass());
        }    
        // Intialization of controllers
        if (parkingEntrancePanelControllerInstance == null)
            parkingEntrancePanelControllerInstance = getServiceInstance(getParkingEntrancePanelControllerClass());
        if (cityEntrancePanelControllerInstance == null)
            cityEntrancePanelControllerInstance = getServiceInstance(getCityEntrancePanelControllerClass());
        if (messengerControllerInstance == null)
            messengerControllerInstance = getServiceInstance(getMessengerControllerClass());
        // Deploying contexts
        deploy(parkingAvailabilityInstance);
        deploy(parkingUsagePatternInstance);
        deploy(averageOccupancyInstance);
        deploy(parkingSuggestionInstance);
        // Deploying controllers
        deploy(parkingEntrancePanelControllerInstance);
        deploy(cityEntrancePanelControllerInstance);
        deploy(messengerControllerInstance);
    }
    
    @Override
    public void undeployAll() {
        // Undeploying contexts
        undeploy(parkingAvailabilityInstance);
        undeploy(parkingUsagePatternInstance);
        undeploy(averageOccupancyInstance);
        undeploy(parkingSuggestionInstance);
        // Undeploying controllers
        undeploy(parkingEntrancePanelControllerInstance);
        undeploy(cityEntrancePanelControllerInstance);
        undeploy(messengerControllerInstance);
    }
    // Abstract binding methods for contexts
    
    public Class<? extends DeviceSourceBuffer> getPresenceFromPresenceSensorBufferClass() {
    	return PresenceBufferForPresenceSensor.class;
    }
    
	
	public Class<? extends Tool> getParkingAvailabilityJobClass() {
		return ParkingAvailabilityJob.class;
	}
	
	public void setParkingAvailabilityMapReduceConfig() {
		MapReduceConfiguration ParkingAvailabilityMapReduce = new MapReduceConfiguration.Builder()
				.contexJobClass(getParkingAvailabilityJobClass())
				.contexClass(getParkingAvailabilityClass()).period(30)
				.timeUnit(TimeUnit.S).build();

		parkingAvailabilityInstance.addToOutProperties(
			ServiceProperties.MAPREDUCE.toString(),
			ParkingAvailabilityMapReduce);
	}
	
	public Class<? extends Tool> getAverageOccupancyJobClass() {
		return AverageOccupancyJob.class;
	}
	
	public void setAverageOccupancyMapReduceConfig() {
		MapReduceConfiguration AverageOccupancyMapReduce = new MapReduceConfiguration.Builder()
				.contexJobClass(getAverageOccupancyJobClass())
				.contexClass(getAverageOccupancyClass()).period(2)
				.timeUnit(TimeUnit.MIN).build();

		averageOccupancyInstance.addToOutProperties(
			ServiceProperties.MAPREDUCE.toString(),
			AverageOccupancyMapReduce);
	}
    public abstract Class<? extends AbstractParkingAvailability> getParkingAvailabilityClass();
    
    public abstract Class<? extends AbstractParkingUsagePattern> getParkingUsagePatternClass();
    
    public abstract Class<? extends AbstractAverageOccupancy> getAverageOccupancyClass();
    
    public abstract Class<? extends AbstractParkingSuggestion> getParkingSuggestionClass();
    
    // End of abstract binding methods for contexts
    
    // Abstract binding methods for controllers
    public abstract Class<? extends AbstractParkingEntrancePanelController> getParkingEntrancePanelControllerClass();
    
    public abstract Class<? extends AbstractCityEntrancePanelController> getCityEntrancePanelControllerClass();
    
    public abstract Class<? extends AbstractMessengerController> getMessengerControllerClass();
    
    // End of abstract binding methods for controllers
}
