package fr.inria.phoenix.diasuite.framework.deployment;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.HashMap;

import fr.inria.diagen.core.service.buffer.Buffer;
import fr.inria.diagen.core.service.buffer.CircularFifoBuffer;
import fr.inria.diagen.core.service.buffer.DeviceSourceBuffer;

import java.lang.Boolean;
import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;

public class PresenceBufferForPresenceSensor implements DeviceSourceBuffer {
	
	private Map<String, Buffer<Boolean>> idToPresenceMap;
	private Map<String, Map<String, Object>> idToAttributes;
	private Map<ParkingLotEnum, Set<String>> parkingLotToIdsMap;	
	private static final int CAPACITY = 12;
	
	public PresenceBufferForPresenceSensor() {
		idToPresenceMap = new HashMap<String, Buffer<Boolean>>();
		idToAttributes = new HashMap<String, Map<String, Object>>();
		parkingLotToIdsMap = new HashMap<ParkingLotEnum, Set<String>>();
	}
	
	@Override
	public void add(Map<String, Object> attributes, Object sourceValue) {
		
		String id = (String) attributes.get("id");
		ParkingLotEnum parkingLot = (ParkingLotEnum) attributes.get("parkingLot");
		
		Set<String> idSet = new HashSet<String>();
		if (id != null  && parkingLot != null) {
			Buffer<Boolean> sourceBuffer;
		
		
			if ((sourceBuffer = idToPresenceMap.get(id)) == null) {
				sourceBuffer = new CircularFifoBuffer<Boolean>(
					CAPACITY);
				idToPresenceMap.put(id, sourceBuffer);
			}
			sourceBuffer.add((Boolean) sourceValue);
			
			Map<String, Object> attributeMap;
			if ((attributeMap = idToAttributes.get(id)) == null) {
				attributeMap = new HashMap<String, Object>();
				idToAttributes.put(id, attributeMap);
			}
			attributeMap.putAll(attributes);
			
if ((idSet = parkingLotToIdsMap.get(parkingLot)) == null) {
	idSet = new HashSet<String>();
	parkingLotToIdsMap.put(parkingLot, idSet);
}
			idSet.add(id);
		
		} else
			throw new NullPointerException(
						"The provided attributes can't be null !");
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer();

		for (String id : idToPresenceMap.keySet()) {
			buffer.append(id + ": [");
			buffer.append(idToPresenceMap.get(id).toString() + "]");
			buffer.append("\n");
		}

	return buffer.toString();
	}
	
	@Override
	public int size() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public Object getContent() {
		return getContentGroupedByAttribute("id", CAPACITY);
	}

	@Override
	public Object getContentGroupedByAttribute(String attribute, int size) {
		switch (attribute) {
		case "parkingLot": {
		if(size == 1) {
			Map<ParkingLotEnum, List<Boolean>> presenceByParkingLotAttributeMap = new HashMap<ParkingLotEnum, List<Boolean>>();
			List<Boolean> presenceListForParkingLot;
			
			for (Entry<ParkingLotEnum, Set<String>> parkingLotDevices : parkingLotToIdsMap
				.entrySet()) {
				presenceListForParkingLot = new ArrayList<Boolean>();
				
				for (String deviceId : parkingLotDevices.getValue()) {
					presenceListForParkingLot = idToPresenceMap.get(deviceId)
							.getLastElement().toList();
				}
				presenceByParkingLotAttributeMap.put(parkingLotDevices.getKey(), presenceListForParkingLot);
			}
			
			return presenceByParkingLotAttributeMap;
		} else {
			Map<ParkingLotEnum, List<Buffer<Boolean>>> presenceByParkingLotAttributeMap = new HashMap<ParkingLotEnum, List<Buffer<Boolean>>>();
			List<Buffer<Boolean>> presenceListForParkingLot = new ArrayList<Buffer<Boolean>>();
			
			for (Entry<ParkingLotEnum, Set<String>> parkingLotDevices : parkingLotToIdsMap
				.entrySet()) {
				presenceListForParkingLot = new ArrayList<Buffer<Boolean>>();
				
				for (String deviceId : parkingLotDevices.getValue()) {
					presenceListForParkingLot.add(idToPresenceMap
					.get(deviceId).getLastElements(size));
				}
				presenceByParkingLotAttributeMap.put(parkingLotDevices.getKey(), presenceListForParkingLot);
			}
			
			return presenceByParkingLotAttributeMap;
		}
		
		}
		
		case "id": {
		if (size == 1) {
			Map<String, List<Boolean>> presenceByIdAttributeMap = new HashMap<String, List<Boolean>>();
			List<Boolean> presenceListForId = new ArrayList<Boolean>();
			for (Entry<String, Buffer<Boolean>> devicePresence : idToPresenceMap
				.entrySet()) {
				presenceListForId = devicePresence.getValue().getLastElement().toList();
				presenceByIdAttributeMap.put(devicePresence.getKey(), presenceListForId);
			}
			
			return presenceByIdAttributeMap;
			
		} else {
			Map<String, Buffer<Boolean>> presenceByIdAttributeMap = new HashMap<String, Buffer<Boolean>>();
			for (Entry<String, Buffer<Boolean>> devicePresence : idToPresenceMap
				.entrySet()) {
				presenceByIdAttributeMap.put(devicePresence.getKey(), 
						devicePresence.getValue().getLastElements(size));
			}
			
			return presenceByIdAttributeMap;
			
			}
		}

		default:
			try {
				throw new Exception(attribute + " is not a valid grouping attribute !");
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		return null;
	}

	@Override
	public void saveContentsToFile(String fileName, int size) {
		try {
			FileWriter fileWritter = new FileWriter(fileName, true);
			BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
			
			StringBuilder builder = new StringBuilder();
			
			for (Entry<String, Buffer<Boolean>> presenceBuffer : idToPresenceMap
					.entrySet()) {
				int i = 1;
				for (Boolean presence : presenceBuffer.getValue().getLastElements(size).toList()) {
					builder.append("{");
					builder.append("\"presence\" : " + "\""
					+ presence.toString() + "\"");

				for (Entry<String, Object> attributeEntry : idToAttributes
					.get(presenceBuffer.getKey()).entrySet())
					builder.append(", \"" + attributeEntry.getKey()
						+ "\": " + "\"" + attributeEntry.getValue()
						+ "\"");

				builder.append(", \"position\" : " + "\"" + i++ + "\"");
				builder.append("}");
				builder.append("\n");
				}
			}

			bufferWritter.write(builder.toString());

			builder.setLength(0);
			bufferWritter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public int capacity() {
		return CAPACITY;
	}
}
