package fr.inria.phoenix.diasuite.framework.device.presencesensor.source;

import java.io.Serializable;

import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

public final class PresenceFromPresenceSensor implements Serializable {
    private static final long serialVersionUID = 0;
    
    private java.lang.Boolean value;
    
    /**
     * Get the value of the source <code>presence</code>
     * 
     * @return the value of the source <code>presence</code>
     */
    public java.lang.Boolean value() {
        return value;
    }
    
    public PresenceFromPresenceSensor(Service service, RemoteServiceInfo remoteServiceInfo, java.lang.Boolean value) {
        this.sender = new PresenceSensorProxy(service, remoteServiceInfo);
        this.value = value;
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        PresenceFromPresenceSensor other = (PresenceFromPresenceSensor) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        return true;
    }
    // Proxy to the sender, i.e., the device that published the source
    private PresenceSensorProxy sender;
    
    /**
     * Get the sender of the source <code>presence</code>, i.e., the <code>PresenceSensor</code> device that published the source.
     * 
     * @return A proxy to the <code>PresenceSensor</code> that triggered the source
     */
    public PresenceSensorProxy sender() {
        return sender;
    }
    
    /**
     * A proxy to the <code>PresenceSensor</code> device enclosing the subscription/unsubscription methods.
     */
    public class PresenceSensorProxy extends Proxy {
        private PresenceSensorProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subcribes to publication of the <code>presence</code> source from the <code>PresenceSensor</code> device.
         */
        public void subscribeToPresence() {
            getService().getProcessor().subscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "presence");
        }
        
        /**
         * Unsubcribes from publication of the <code>presence</code> source from the <code>PresenceSensor</code> device.
         */
        public void unsubscribePresence() {
            getService().getProcessor().unsubscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "presence");
        }
        
        public fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum parkingLot() {
            return (fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum) callGetValue("parkingLot");
        }
        
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
}
