package parkingmanager.context;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import parkingmanager.enumeration.ParkingLotSizeEnum;
import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.mapreduce.MapCollector;
import fr.inria.diagen.core.mapreduce.ReduceCollector;
import fr.inria.phoenix.diasuite.framework.context.averageoccupancy.AbstractAverageOccupancy;
import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;
import fr.inria.phoenix.diasuite.framework.datatype.parkingoccupancy.ParkingOccupancy;
import fr.inria.phoenix.diasuite.framework.datatype.presence.Presence;

public class AverageOccupancy extends AbstractAverageOccupancy {

	public AverageOccupancy(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);
	}

	@Override
	public void map(ParkingLotEnum key, Boolean value,
			MapCollector<ParkingLotEnum, Presence> collector, String... args) {
		if (!value) {
			collector.emitMap(key, new Presence(true, String.valueOf(args[0])));
		}
	}

	@Override
	public void reduce(ParkingLotEnum key, List<Presence> values,
			ReduceCollector<ParkingLotEnum, Integer> collector) {
		Map<Integer, List<Boolean>> availabilityPerTimeStamp = new HashMap<Integer, List<Boolean>>();
		Map<Integer, Float> averageAvailabilityPerTimeStamp = new HashMap<Integer, Float>();

		availabilityPerTimeStamp = transformPresenceListToMap(values);

		for (Entry<Integer, List<Boolean>> availabilityEntry : availabilityPerTimeStamp
				.entrySet()) {
			int sum = 0;
			for (int i = 0; i < availabilityEntry.getValue().size(); i++) {
				sum++;
			}

			int parkingLotSize = ParkingLotSizeEnum.valueOf(key.name())
					.getSize();
			Float averageAvailability = new Float(
					((float) sum / parkingLotSize) * 100);

			averageAvailabilityPerTimeStamp.put(availabilityEntry.getKey(),
					averageAvailability);
		}

		Float simpleAverageSum = new Float(0);
		int n = averageAvailabilityPerTimeStamp.size();
		for (Entry<Integer, Float> averageAvailability : averageAvailabilityPerTimeStamp
				.entrySet())
			simpleAverageSum = simpleAverageSum
					+ averageAvailability.getValue();

		collector.emitReduce(key, Math.round(simpleAverageSum / n));
	}

	/**
	 * @param values
	 * @return
	 */
	public Map<Integer, List<Boolean>> transformPresenceListToMap(
			List<Presence> values) {
		Map<Integer, List<Boolean>> map = new HashMap<Integer, List<Boolean>>();

		for (Presence presence : values) {
			Boolean presenceValue = presence.getPresence();
			Integer presenceValueArrivalOrder = Integer.parseInt(presence
					.getTime());

			if (!map.containsKey(presenceValueArrivalOrder))
				map.put(presenceValueArrivalOrder, new ArrayList<Boolean>());

			map.get(presenceValueArrivalOrder).add(presenceValue);
		}

		return map;
	}

	@Override
	protected List<ParkingOccupancy> onPeriodicPresenceFromPresenceSensor(
			Map<ParkingLotEnum, Integer> presenceByParkingLotMap) {

		List<ParkingOccupancy> parkingOccupancyList = new ArrayList<ParkingOccupancy>();

		for (Entry<ParkingLotEnum, Integer> entry : presenceByParkingLotMap
				.entrySet()) {
			parkingOccupancyList.add(new ParkingOccupancy(entry.getKey(),
					new Float((float) entry.getValue())));
		}

		return parkingOccupancyList;
	}
}
