package parkingmanager.context;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.mapreduce.MapCollector;
import fr.inria.diagen.core.mapreduce.ReduceCollector;
import fr.inria.phoenix.diasuite.framework.context.parkingavailability.AbstractParkingAvailability;
import fr.inria.phoenix.diasuite.framework.datatype.availability.Availability;
import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;

public class ParkingAvailability extends AbstractParkingAvailability {

	public ParkingAvailability(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);
	}

	@Override
	public void map(ParkingLotEnum parkingLot, Boolean presence,
			MapCollector<ParkingLotEnum, Boolean> collector, String... args) {
		if (!presence)
			collector.emitMap(parkingLot, true);
	}

	@Override
	public void reduce(ParkingLotEnum parkingLot, List<Boolean> values,
			ReduceCollector<ParkingLotEnum, Integer> collector) {
		int sum = 0;
		for (int i = 0; i < values.size(); i++)
			sum++;

		collector.emitReduce(parkingLot, sum);
	}

	@Override
	protected List<Availability> onPeriodicPresenceFromPresenceSensor(
			Map<ParkingLotEnum, Integer> presenceByParkingLotMap) {
		List<Availability> availabilityList = new ArrayList<Availability>();

		if (presenceByParkingLotMap.size() != 0)
			for (Entry<ParkingLotEnum, Integer> parkingLot : presenceByParkingLotMap
					.entrySet()) {
				Availability availability = new Availability(
						parkingLot.getKey(), parkingLot.getValue());
				availabilityList.add(availability);
			}
		else
			for (ParkingLotEnum parkingLot : ParkingLotEnum.values())
				availabilityList.add(new Availability(parkingLot, 0));

		return availabilityList;
	}
}
